package maze;

/** A class to represent cells in a maze. Cells can be blocked,
 * open, or visited. In addition, entrances and exits in a maze
 * are represented by a special cell type.
 */
public class Cell {
  /* Class variables */
  private static final char OPEN = 'o';
  private static final char VISITED = 'v';
  private static final char CLOSED = '#';
  private static final char ENTRANCE = 'e';
  private static final char EXIT = 'x';
  
  /* Member variables. */
  private int row;
  private int column;
  private char state;
  
  /* Constructor */
  public Cell(int row,int column,char code) {
    this.row = row;
    this.column = column;
    state = code;
  }
  
  /* Accessor methods */
  public boolean isOpen() { return state == OPEN || state == EXIT; }
  public boolean isEntrance() { return state == ENTRANCE; }
  public boolean isExit() { return state == EXIT; }
  public int getRow() { return row; }
  public int getColumn() { return column; }
  public char toChar() {
    return state;
   }
  public String toString() {
    return "(" + row + "," + column + ":" + toChar() + ")";
  }
  
  /* Mutator methods */
  public void markVisited() {
    if(state == OPEN) 
      state = VISITED;
  }
  public void clearVisited() {
    if(state == VISITED)
      state = OPEN;
  }

  /* Test program to test the Cell class. */
  public static void main(String[] args) {
    Cell firstCell = new Cell(1,1,'o');
    System.out.println("Before visit the cell is: " + firstCell.toString());
    firstCell.markVisited();
    System.out.println("After visit the cell is: " + firstCell.toString());
    
    System.out.println();
    
    Cell secondCell = new Cell(3,1,'#');
    System.out.println("Before visit the cell is: " + secondCell.toString());
    firstCell.markVisited();
    System.out.println("After visit the cell is: " + secondCell.toString());
  }
}
