package maze;

import java.io.File;
import java.util.Scanner;

public class Maze {
  /* Member variables */
  private Cell[][] grid;
  
  /* Construct a maze by reading it from a file. */
  public Maze(String fileName)
  {
    int rows, columns;
    Scanner input = null;
    try {
        input = new Scanner(new File(fileName));
    } catch(Exception ex) {
        ex.printStackTrace();
        System.exit(-1);
    }
    
    rows = input.nextInt();
    columns = input.nextInt();
    
    grid = new Cell[rows][columns];
    for(int row = 0;row < rows;row++) {
      String theRow = input.next();
      for(int column = 0;column < columns;column++) {
        char theChar = theRow.charAt(column);
        Cell newCell = new Cell(row,column,theChar);
        grid[row][column] = newCell;
      }
    }
  }
  
  /* Accessor methods */
  
  // Find and return an entrance cell
  public Cell findEntrance()
  {
    int rows = grid.length;
    int cols = grid[0].length;
    for(int r = 0;r < rows;r++) {
        for(int c = 0; c < cols; c++) {
            if(grid[r][c].isEntrance())
                return grid[r][c];
        }
    }
    return null;
  }
  
  // Find a neighbor of the given cell that is open
  // and return a reference to that neighbor cell. If
  // you can find no such neighbor, return null.
    public Cell findOpenNeighbor(Cell fromCell)
    {
      int row = fromCell.getRow();
      int col = fromCell.getColumn();
      if(row - 1 >= 0 && grid[row-1][col].isOpen())
          return grid[row-1][col];
      if(col - 1 >= 0 && grid[row][col-1].isOpen())
          return grid[row][col-1];
      if(row + 1 < grid.length && grid[row+1][col].isOpen())
          return grid[row+1][col];
      if(col + 1 < grid[row].length && grid[row][col+1].isOpen())
          return grid[row][col+1];
      return null;
    }
  
  // Print the contents of the Maze to System.out
  public void print() {
    for(int row = 0;row < grid.length;row++) {
      for(int col = 0;col < grid[row].length;col++)
        System.out.print(grid[row][col].toChar());
      System.out.println();
    }
  }
  
  /* Mutator methods */
  
  // Reset all visited cells to open
  public void clearVisited() {
    int rows = grid.length;
    int cols = grid[0].length;
    for(int r = 0;r < rows;r++) {
        for(int c = 0; c < cols; c++) {
            grid[r][c].clearVisited();
        }
    }
  }
  
  // Test program to test the Maze class
    public static void main(String args[]) throws Exception {
      Maze m = new Maze("test.maze");

      m.print();

      System.out.println();

      Cell entrance = m.findEntrance();
      System.out.println("Entrance is " + entrance);
      Cell neighbor = m.findOpenNeighbor(entrance);
      System.out.println("Neighbor is " + neighbor);
    }
}
